/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package net.sf.xradar.util;

import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.List;

/**
 * strongly inspired by:
 * http://pmd.svn.sourceforge.net/viewvc/pmd/trunk/pmd/src/net/sourceforge/pmd/util/FileFinder.java?view=log
 */

/**
 * A utility class for finding files within some directories.
 */
public class FileFinder {

    public FileFinder() 
    {

	}

	private FilenameFilter filter;
    static final String FILE_SEP = System.getProperty("file.separator");

    public ArrayList<File> findFilesFrom(String dir, FilenameFilter filter, boolean recurse) 
    {
        this.filter = filter;
        ArrayList<File> files = new ArrayList<File>();
        
        scanDirectory(new File(dir), files, recurse);
        return files;
    }
    /**
     * Implements a tail recursive file scanner
     */
    private void scanDirectory(File dir, List<File> list, boolean recurse) 
    {
    	
    	FilterDir filterDir = new FilterDir();
    	String[] candidates = dir.list(filter);
        String[] notCandidates = dir.list(filterDir);

        if (candidates == null) 
        {
            return;
        }
    	else if (candidates.length==0)
        {

    		for (int i = 0; i < notCandidates.length; i++) 
    		{	
    			File tmp2 = new File(dir + FILE_SEP +notCandidates[i]);
    			if (recurse) 
    			{
    				scanDirectory(tmp2, list, true);
    			}
    	        
    		}
        }
    	else if (notCandidates.length!=0)
        {
        	for (int j = 0; j < candidates.length; j++) 
	        {
	            File tmp = new File(dir + FILE_SEP + candidates[j]);

	            if (tmp.isDirectory())
	            {
	                if (recurse) 
	                {
	                    scanDirectory(tmp, list, true);
	                }
	            }
	            else 
	            {

	                list.add(new File(dir + FILE_SEP + candidates[j]));

	            }
	            
	        }
        }

    }
    

    
}
